@@ -16,6 +16,8 @@
 
 from __future__ import print_function
 
+from . import hooks
+
 import argparse
 import configparser
 import datetime
@@ -24,16 +26,25 @@ import json
 import os
 import re
 import shlex
+import stat
 import subprocess
 import sys
 import textwrap
+from urllib.parse import quote_plus
 from urllib.parse import urlencode
 from urllib.parse import urljoin
 from urllib.parse import urlparse
 
-import pkg_resources
 import requests
 
+try:
+    import importlib.metadata as importlib_metadata
+    pkg_resources = None
+except ImportError:
+    # Pre-py38
+    importlib_metadata = None
+    import pkg_resources
+
 
 VERBOSE = False
 UPDATE = False
@@ -219,9 +230,12 @@ def run_http_exc(klazz, url, **env):
 
 
 def get_version():
-    requirement = pkg_resources.Requirement.parse('git-review')
-    provider = pkg_resources.get_provider(requirement)
-    return provider.version
+    if importlib_metadata:
+        return importlib_metadata.version('git-review')
+    else:
+        requirement = pkg_resources.Requirement.parse('git-review')
+        provider = pkg_resources.get_provider(requirement)
+        return provider.version
 
 
 def get_git_version():
@@ -383,7 +397,10 @@ def set_hooks_commit_msg(remote, target_file):
 
     if not os.path.exists(target_file) or UPDATE:
         remote_url = get_remote_url(remote)
-        if (remote_url.startswith('http://') or
+        if not remote_url:
+            with open(target_file, 'w') as f:
+                f.write(hooks.COMMIT_MSG)
+        elif (remote_url.startswith('http://') or
                 remote_url.startswith('https://')):
             hook_url = urljoin(remote_url, '/tools/hooks/commit-msg')
             if VERBOSE:
@@ -428,8 +445,17 @@ def set_hooks_commit_msg(remote, target_file):
             "git", "submodule", "foreach",
             'cp -p %s "$(git rev-parse --git-dir)/hooks/"' % target_file)
 
-    if not os.access(target_file, os.X_OK):
-        os.chmod(target_file, os.path.stat.S_IREAD | os.path.stat.S_IEXEC)
+    old_mask = stat.S_IMODE(os.stat(target_file).st_mode)
+    # make sure the owner always has read+write+exec perms
+    mask = old_mask | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
+    # add executable permission for everyone else who has read access
+    if mask | stat.S_IRGRP == mask:
+        mask |= stat.S_IXGRP
+    if mask | stat.S_IROTH == mask:
+        mask |= stat.S_IXOTH
+    # only call chmod if we need to change the mask
+    if mask != old_mask:
+        os.chmod(target_file, mask)
 
 
 def test_remote_url(remote_url):
@@ -564,6 +590,11 @@ def get_remote_url(remote):
     URL rather than the fetch URL.
     """
 
+    # Short-circuit to a false value if the remote passed in is falsey, used
+    # for determining commit-msg hook retrieval behavior
+    if not remote:
+        return remote
+
     push_url = git_config_get_value('remote.%s' % remote, 'pushurl')
     if push_url is not None:
         # Git rewrites pushurl using insteadOf but not pushInsteadOf.
@@ -719,7 +750,7 @@ def query_reviews_over_ssh(remote_url, project=None, change=None,
         print("Query gerrit %s %s" % (remote_url, query))
     output = run_command_exc(
         exception,
-        "ssh", "-x" + port_data, userhost,
+        os.environ.get("GIT_SSH", "ssh"), "-x" + port_data, userhost,
         "gerrit", "query",
         "--format=JSON %s" % query)
     if VERBOSE:
@@ -900,7 +931,7 @@ def check_remote(branch, remote, scheme, hostname, port, project,
         raise
 
 
-def rebase_changes(branch, remote, interactive=True):
+def rebase_changes(branch, remote, interactive=True, keep=False):
 
     global _orig_head
 
@@ -959,17 +990,37 @@ def rebase_changes(branch, remote, interactive=True):
         print("Errors running %s" % cmd)
         if interactive:
             print(output)
-            printwrap("It is likely that your change has a merge conflict. "
-                      "You may resolve it in the working tree now as "
-                      "described above and then run 'git review' again, or "
-                      "if you do not want to resolve it yet (note that the "
-                      "change can not merge until the conflict is resolved) "
-                      "you may run 'git rebase --abort' then 'git review -R' "
-                      "to upload the change without rebasing.")
+            if keep:
+                printwrap("It is likely that your change has a merge "
+                          "conflict. You may resolve it in the working tree "
+                          "now as described above and then run 'git review' "
+                          "again, or if you do not want to resolve it yet "
+                          "(note that the change can not merge until the "
+                          "conflict is resolved) you may run 'git rebase "
+                          "--abort' then 'git review -R' to upload the change "
+                          "without rebasing.")
+            else:
+                printwrap("It is likely that your change has a merge "
+                          "conflict, but the result of the test rebase has "
+                          "been discarded. You may rebase it yourself and "
+                          "review again, or use 'git review -R' to upload the "
+                          "change without resolving the conflict (though this "
+                          "is discouraged since it probably can't be merged "
+                          "in that state), or use the --keep-rebase option to "
+                          "leave the incomplete rebase result in your working "
+                          "tree.")
         return False
     return True
 
 
+def abort_rebase():
+    cmd = "git rebase --abort"
+    (status, output) = run_command_status(cmd)
+    if status != 0:
+        print("Errors running %s" % cmd)
+        print(output)
+
+
 def undo_rebase():
     global _orig_head
     if not _orig_head:
@@ -1015,9 +1066,18 @@ def assert_one_change(remote, branch, yes, have_hook):
                   "installed. Amending the commit to add a gerrit change id.")
         run_command("git commit --amend", GIT_EDITOR='true')
     elif output_lines == 0:
-        printwrap("No changes between HEAD and %s/%s. Submitting for review "
+        printwrap("The commit HEAD already exists on a branch on the remote. "
+                  "If it already exists on %s/%s, then submitting for review "
                   "would be pointless." % (remote, branch))
-        sys.exit(1)
+        print("\n\n"
+              "Do you really want to submit this review?")
+        try:
+            yes_no = input("Type 'yes' to confirm, other to cancel: ")
+        except KeyboardInterrupt:
+            yes_no = "no"
+        if yes_no.lower().strip() != "yes":
+            print("Aborting.")
+            sys.exit(1)
     elif output_lines > 1:
         if not yes:
             printwrap("You are about to submit multiple commits. This is "
@@ -1435,14 +1495,28 @@ class MalformedInput(GitReviewException):
     EXIT_CODE = 3
 
 
+def assert_valid_hashtags(hashtags):
+    """Ensure no whitespace is found in hashtags, as it will result
+    in an invalid refspec.
+    """
+    assert_valid_whitespace(hashtags, "hashtags")
+
+
 def assert_valid_reviewers(reviewers):
     """Ensure no whitespace is found in reviewer names, as it will result
     in an invalid refspec.
     """
-    for reviewer in reviewers:
-        if re.search(r'\s', reviewer):
+    assert_valid_whitespace(reviewers, "reviewers")
+
+
+def assert_valid_whitespace(values, type_name):
+    """Ensure no whitespace is found in list values, as it will result
+    in an invalid refspec.
+    """
+    for v in values:
+        if re.search(r"\s", v):
             raise MalformedInput(
-                "Whitespace not allowed in reviewer: '%s'" % reviewer)
+                "Whitespace not allowed in %s: '%s'" % type_name, v)
 
 
 class _DownloadFlag(argparse.Action):
@@ -1519,8 +1593,10 @@ additional information:
                                  action="store_true",
                                  help="No topic except if explicitly provided")
 
+    parser.add_argument("--hashtags", nargs="+",
+                        help="Hashtags to submit branch to")
     parser.add_argument("--reviewers", nargs="+",
-                        help="Add reviewers to uploaded patch sets.")
+                        help="Add reviewers to uploaded patchsets.")
     parser.add_argument("-n", "--dry-run", dest="dry", action="store_true",
                         help="Don't actually submit the branch for review")
     parser.add_argument("-i", "--new-changeid", dest="regenerate",
@@ -1528,6 +1604,9 @@ additional information:
                         help="Regenerate Change-id before submitting")
     parser.add_argument("-r", "--remote", dest="remote",
                         help="git remote to use for gerrit")
+    parser.add_argument("--remote-hook", dest="remote_hook",
+                        action="store_true",
+                        help="Fetch the remote version of the commit-msg hook")
     parser.add_argument("--use-pushurl", dest="usepushurl",
                         action="store_true",
                         help="Use remote push-url logic instead of separate"
@@ -1536,14 +1615,22 @@ additional information:
                         choices=['NONE', 'OWNER', 'OWNER_REVIEWERS', 'ALL'],
                         help="Control to whom email notifications are sent,"
                              " defaults to ALL.")
+    parser.add_argument("--message", dest="message",
+                        help="Message to add to patchset description")
 
     rebase_group = parser.add_mutually_exclusive_group()
     rebase_group.add_argument("-R", "--no-rebase", dest="rebase",
                               action="store_false",
-                              help="Don't rebase changes before submitting.")
+                              help="Don't test for remote merge conflicts"
+                                   " before pushing.")
     rebase_group.add_argument("-F", "--force-rebase", dest="force_rebase",
                               action="store_true",
-                              help="Force rebase even when not needed.")
+                              help="Force and push a rebase even when not"
+                                   " needed.")
+    rebase_group.add_argument("-K", "--keep-rebase", dest="keep_rebase",
+                              action="store_true",
+                              help="Keep the unfinished test rebase if a "
+                                   " merge conflict is detected.")
 
     track_group = parser.add_mutually_exclusive_group()
     track_group.add_argument("--track", dest="track",
@@ -1613,7 +1700,7 @@ additional information:
                         help="Output more information about what's going on")
 
     wip_group = parser.add_mutually_exclusive_group()
-    wip_group.add_argument("-w", "--work-in-progress", dest="wip",
+    wip_group.add_argument("-w", "--wip", "--work-in-progress", dest="wip",
                            action="store_true",
                            help="Send patch as work in progress for Gerrit "
                                 "versions >= 2.15")
@@ -1758,7 +1845,10 @@ additional information:
     have_hook = os.path.exists(hook_file) and os.access(hook_file, os.X_OK)
 
     if not have_hook:
-        set_hooks_commit_msg(remote, hook_file)
+        if options.remote_hook:
+            set_hooks_commit_msg(remote, hook_file)
+        else:
+            set_hooks_commit_msg(None, hook_file)
 
     if options.setup:
         if options.finish and not options.dry:
@@ -1766,7 +1856,9 @@ additional information:
         return
 
     if options.rebase or options.force_rebase:
-        if not rebase_changes(branch, remote):
+        if not rebase_changes(branch, remote, keep=options.keep_rebase):
+            if not options.keep_rebase:
+                abort_rebase()
             sys.exit(1)
         if not options.force_rebase and not undo_rebase():
             sys.exit(1)
@@ -1789,6 +1881,10 @@ additional information:
     if topic and topic != branch:
         push_options.append("topic=%s" % topic)
 
+    if options.hashtags:
+        assert_valid_hashtags(options.hashtags)
+        push_options += ["t=%s" % r for r in options.hashtags]
+
     if options.reviewers:
         assert_valid_reviewers(options.reviewers)
         push_options += ["r=%s" % r for r in options.reviewers]
@@ -1819,6 +1915,10 @@ additional information:
     if options.notify is not None:
         push_options.append("notify=%s" % options.notify)
 
+    if options.message is not None:
+        escaped_message = quote_plus(options.message)
+        push_options.append(f"m={escaped_message}")
+
     if push_options:
         cmd += "%" + ",".join(push_options)
     if options.dry:
