@@ -37,10 +37,12 @@ else:
     urlparse = urllib.parse.urlparse
 
 
-WAR_URL = 'https://gerrit-releases.storage.googleapis.com/gerrit-2.13.14.war'
+DEFAULT_WAR_URL = 'https://gerrit-releases.storage.googleapis.com/' \
+                  'gerrit-3.4.4.war'
 # Update GOLDEN_SITE_VER for every change altering golden site, including
-# WAR_URL changes. Set new value to something unique (just +1 it for example)
-GOLDEN_SITE_VER = '5'
+# DEFAULT_WAR_URL changes. Set new value to something unique (just +1 it for
+# example).
+GOLDEN_SITE_VER = '7'
 
 
 # NOTE(yorik-sar): This function needs to be a perfect hash function for
@@ -108,12 +110,12 @@ class IsoEnvDir(DirHelpers, fixtures.Fixture):
 
 
 class GerritHelpers(DirHelpers):
-
     def init_dirs(self):
         self.primary_dir = os.path.abspath(os.path.curdir)
         self.gerrit_dir = self._dir('primary', '.gerrit')
         self.gsite_dir = self._dir('gerrit', 'golden_site')
-        self.gerrit_war = self._dir('gerrit', WAR_URL.split('/')[-1])
+        self.war_url = os.environ.get('WAR_URL', DEFAULT_WAR_URL)
+        self.gerrit_war = self._dir('gerrit', self.war_url.split('/')[-1])
 
     def ensure_gerrit_war(self):
         # check if gerrit.war file exists in .gerrit directory
@@ -121,8 +123,8 @@ class GerritHelpers(DirHelpers):
             os.mkdir(self.gerrit_dir)
 
         if not os.path.exists(self.gerrit_war):
-            print("Downloading Gerrit binary from %s..." % WAR_URL)
-            resp = requests.get(WAR_URL)
+            print("Downloading Gerrit binary from %s..." % self.war_url)
+            resp = requests.get(self.war_url)
             if resp.status_code != 200:
                 raise RuntimeError("Problem requesting Gerrit war")
             utils.write_to_file(self.gerrit_war, resp.content)
@@ -164,8 +166,9 @@ class GerritHelpers(DirHelpers):
         with open(golden_ver_file, 'w') as f:
             f.write(GOLDEN_SITE_VER)
 
-        # create SSH public key
-        utils.run_cmd('ssh-keygen', '-t', 'rsa', '-b', '4096', '-m', 'PEM',
+        # Create SSH public key. This is not RSA because Jammy wants to
+        # do rsa + sha2 by default and Gerrit 3.4.4 does not understand this.
+        utils.run_cmd('ssh-keygen', '-t', 'ed25519',
                       '-f', self._dir('gsite', 'test_ssh_key'), '-N', '')
 
     def _run_gerrit_cli(self, command, *args):
@@ -225,7 +228,7 @@ class BaseGitReviewTestCase(testtools.TestCase, GerritHelpers):
             pub_key = pub_key_fd.read().decode().strip()
         resp = requests.post(
             'http://%s:%s/a/accounts/self/sshkeys' % (http_addr, http_port),
-            auth=requests.auth.HTTPDigestAuth('admin', 'secret'),
+            auth=requests.auth.HTTPBasicAuth('admin', 'secret'),
             headers={'Content-Type': 'text/plain'},
             data=pub_key)
         if resp.status_code != 201:
