@@ -65,9 +65,16 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
         """Test git-review on the just cloned repository."""
         self._simple_change('test file modified', 'test commit message')
         self.assertNotIn('Change-Id:', self._run_git('log', '-1'))
-        self.assertIn('remote: New Changes:', self._run_git_review())
+        gr = self._run_git_review()
+        self.assertIn('remote: SUCCESS', gr)
+        self.assertIn('test commit message [NEW]', gr)
         self.assertIn('Change-Id:', self._run_git('log', '-1'))
 
+    def test_git_review_version(self):
+        """Test git-review --version."""
+        self.assertTrue(
+            self._run_git_review('--version').startswith('git-review version'))
+
     def test_git_review_s(self):
         """Test git-review -s."""
         self.reset_remote()
@@ -103,6 +110,42 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
         self._run_git_review(
             '-s', chdir=os.path.join(self.test_dir, 'subdirectory'))
 
+    def test_install_embedded_hook(self):
+        """Test whether git-review -s correctly creates the commit-msg hook
+        from the embedded copy with appropriate permissions and content.
+        """
+        hooks_subdir = ".git/hooks"
+        hook_file = os.path.join(self.test_dir, hooks_subdir, 'commit-msg')
+        self.reset_remote()
+
+        self._run_git_review('-s')
+        self.assertTrue(os.path.exists(hook_file))
+        self.assertTrue(os.access(hook_file, os.W_OK))
+        self.assertTrue(os.access(hook_file, os.X_OK))
+        with open(hook_file) as f:
+            content = f.read()
+            self.assertTrue(content.startswith('#!/'))
+            # This line should not appear in the embedded version
+            self.assertNotIn('# From Gerrit Code Review', content)
+
+    def test_install_remote_hook(self):
+        """Test whether git-review -s correctly creates the commit-msg hook
+        from the Gerrit server with appropriate permissions and content.
+        """
+        hooks_subdir = ".git/hooks"
+        hook_file = os.path.join(self.test_dir, hooks_subdir, 'commit-msg')
+        self.reset_remote()
+
+        self._run_git_review('-s', '--remote-hook')
+        self.assertTrue(os.path.exists(hook_file))
+        self.assertTrue(os.access(hook_file, os.W_OK))
+        self.assertTrue(os.access(hook_file, os.X_OK))
+        with open(hook_file) as f:
+            content = f.read()
+            self.assertTrue(content.startswith('#!/'))
+            # This line is injected by the Gerrit server in its version
+            self.assertIn('# From Gerrit Code Review', content)
+
     def test_git_review_s_without_core_hooks_path_option(self):
         """Test whether git-review -s correctly creates the commit-msg hook,
         with the Git core.hooksPath option unset.
@@ -223,7 +266,7 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
 
         review_res = self._run_git_review(confirm=True)
         self.assertIn("Type 'yes' to confirm", review_res)
-        self.assertIn("Processing changes: new: 2", review_res)
+        self.assertIn("new: 2", review_res)
 
         # abandon changes sent to the Gerrit
         head = self._run_git('rev-parse', 'HEAD')
@@ -237,7 +280,10 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
         self._simple_change('test file modified 4th time',
                             'test commit message 4')
         review_res = self._run_git_review('-y')
-        self.assertIn("Processing changes: new: 2", review_res)
+        self.assertIn("new: 2", review_res)
+        self.assertIn("remote: SUCCESS", review_res)
+        self.assertIn("test commit message 3 [NEW]", review_res)
+        self.assertIn("test commit message 4 [NEW]", review_res)
 
     def test_git_review_re(self):
         """Test git-review adding reviewers to changes."""
@@ -253,9 +299,9 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
 
         review_res = self._run_git_review('--reviewers', 'reviewer1',
                                           'reviewer2')
-        self.assertIn("Processing changes: new: 1", review_res)
+        self.assertIn("new: 1", review_res)
 
-        # verify both reviewers are on patch set
+        # verify both reviewers are on patchset
         head = self._run_git('rev-parse', 'HEAD')
         change = self._run_gerrit_cli('query', '--format=JSON',
                                       '--all-reviewers', head)
@@ -328,8 +374,24 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
                    "Errors running git rebase --preserve-merges "
                    "-i remotes/%s/master" % self._remote in exc.args[0])
         self.assertTrue(rebased)
-        self.assertIn("It is likely that your change has a merge conflict.",
+        self.assertIn("It is likely that your change has a merge conflict",
                       exc.args[0])
+        exc = self.assertRaises(Exception, self._run_git, 'rebase', '--abort')
+        self.assertIn("No rebase in progress?", exc.args[0])
+
+    def test_keep_aborted_rebase_state(self):
+        """Test git-review -K behavior when rebase is needed."""
+        self._run_git_review('-s')
+        head_1 = self._run_git('rev-parse', 'HEAD^1')
+
+        self._run_git('checkout', '-b', 'test_branch', head_1)
+
+        self._simple_change('some other message',
+                            'create conflict with master')
+
+        self.assertRaises(Exception, self._run_git_review, '-K')
+        self.assertIn(
+            "interactive rebase in progress", self._run_git('status'))
 
     def test_upload_without_rebase(self):
         """Test change not needing a rebase can upload without rebasing."""
