@@ -105,7 +105,7 @@ class GitReviewConsole(testtools.TestCase, fixtures.TestWithFixtures):
 
         try:
             with mock.patch('sys.argv', ['git-review', '--version']):
-                with self.assertRaisesRegexp(SystemExit, '1'):
+                with self.assertRaisesRegex(SystemExit, '1'):
                     cmd.main()
         except Exception as e:
             self.fail('Exception not expected: %s' % e)
