@@ -18,13 +18,31 @@ import subprocess
 import traceback
 
 
+GERRIT_CONF_TMPL = """
+[gerrit]
+    basePath = git
+    canonicalWebUrl = http://nonexistent/
+[database]
+    type = h2
+    database = db/ReviewDB
+[auth]
+    type = DEVELOPMENT_BECOME_ANY_ACCOUNT
+[sshd]
+    listenAddress = %s:%s
+[httpd]
+    listenUrl = http://%s:%s/
+[sendemail]
+    enable = false
+"""
+
+
 def run_cmd(*args, **kwargs):
     """Run command and check the return code."""
-    preexec_fn = None
-
     if 'chdir' in kwargs:
         def preexec_fn():
             return os.chdir(kwargs['chdir'])
+    else:
+        preexec_fn = None
 
     try:
         proc = subprocess.Popen(args, stdin=subprocess.PIPE,
@@ -64,21 +82,6 @@ def write_to_file(path, content):
     with open(path, 'wb') as file_:
         file_.write(content)
 
-GERRIT_CONF_TMPL = """
-[gerrit]
-    basePath = git
-    canonicalWebUrl = http://nonexistent/
-[database]
-    type = h2
-    database = db/ReviewDB
-[auth]
-    type = DEVELOPMENT_BECOME_ANY_ACCOUNT
-[sshd]
-    listenAddress = %s:%s
-[httpd]
-    listenUrl = http://%s:%s/
-"""
-
 
 def get_gerrit_conf(ssh_addr, ssh_port, http_addr, http_port):
     return GERRIT_CONF_TMPL % (ssh_addr, ssh_port, http_addr, http_port)
